/*
 * mcubecgiin.cpp v 1.01
 * 4x4x4 Rubik's Cube Solver CGI/HTML interface (c) 2005 by Eric Dietz
 * notes: readme.txt  email: root@wrongway.org
 */

// includes
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string>
// namespace
using namespace std;

// local includes
#include "mcube.h"

// shared variables
const char *version = "1.01";
mcube thecube;
string html_template;
string col[7];
int disp_graphics;
int disp_diagrams;
int fixcol;
int adv;
int sel;
int help;
string cmd;
const int N = 4;

// dump javascript defaults into the html template
void java_defaults()
{
  int n = 0;
  printf("ourcub = Array(");
  for (int i = 1; i <= N; i++)
    for (int j = 1; j <= N; j++)
    {
      printf("%i", thecube.cube[j][N+1][N+1-i]);
      n++; if (n < 96) printf(",");
    }
  for (int i = 1; i <= N; i++)
    for (int j = 1; j <= N; j++)
    {
      printf("%i", thecube.cube[0][N+1-i][N+1-j]);
      n++; if (n < 96) printf(",");
    }
  for (int i = 1; i <= N; i++)
    for (int j = 1; j <= N; j++)
    {
      printf("%i", thecube.cube[j][N+1-i][0]);
      n++; if (n < 96) printf(",");
    }
  for (int i = 1; i <= N; i++)
    for (int j = 1; j <= N; j++)
    {
      printf("%i", thecube.cube[N+1][N+1-i][j]);
      n++; if (n < 96) printf(",");
    }
  for (int i = 1; i <= N; i++)
    for (int j = 1; j <= N; j++)
    {
      printf("%i", thecube.cube[N+1-j][N+1-i][N+1]);
      n++; if (n < 96) printf(",");
    }
  for (int i = 1; i <= N; i++)
    for (int j = 1; j <= N; j++)
    {
      printf("%i", thecube.cube[j][0][i]);
      n++; if (n < 96) printf(",");
    }
  printf(");\n");
  printf("ourcol = Array( \"0\" ");
  for (int i = 1; i <= 6; i++)
    printf(", \"%s\" ", col[i].c_str());
  printf(");\n");
  printf("disp_graphics = %i;\n", disp_graphics);
  printf("disp_diagrams = %i;\n", disp_diagrams);
  printf("fixcol = %i;\n", fixcol);
  printf("adv = %i;\n", adv);
  printf("sel = %i;\n", sel);
  printf("help = %i;\n", help);
  //printf("ver = \"%s (%s)\";\n", version, mcube::version);
}

// read optional command-line
void readcommandline(string cmdln)
{
  string cmd = cmdln;
  int n;
  string clr[7];
  srand(time(NULL));
  html_template = "mcubecgiin.htm";
  thecube.resetcube();
  col[1] = "FFFFFF"; col[2] = "0000FF"; col[3] = "FF8000";
  col[4] = "00FF00"; col[5] = "FF0000"; col[6] = "FFFF00";
  disp_graphics = 0;
  disp_diagrams = 0;
  fixcol = 0;
  adv = 0;
  sel = 1;
  help = 0;
  while (cmd.length() > 0)
  {
    if (cmd.substr(0, 9) == "!scrambl1")
    {
      cmd = cmd.substr(9, cmd.length() - 9);
      thecube.scramblecube();
    }
    else if (cmd.substr(0, 6) == "!cube-" && cmd.length() >= (N*N*6+6))
    {
      cmd = cmd.substr(6, cmd.length() - 6);
      for (int i = 1; i <= N; i++)
      {
        for (int j = 1; j <= N; j++)
        {
          thecube.cube[j][N+1][N+1-i]     = cmd.at(((i-1)*N+(j-1))      )-'0';
          thecube.cube[0][N+1-i][N+1-j]   = cmd.at(((i-1)*N+(j-1))+N*N  )-'0';
          thecube.cube[j][N+1-i][0]       = cmd.at(((i-1)*N+(j-1))+N*N*2)-'0';
          thecube.cube[N+1][N+1-i][j]     = cmd.at(((i-1)*N+(j-1))+N*N*3)-'0';
          thecube.cube[N+1-j][N+1-i][N+1] = cmd.at(((i-1)*N+(j-1))+N*N*4)-'0';
          thecube.cube[j][0][i]           = cmd.at(((i-1)*N+(j-1))+N*N*5)-'0';
        }
      }
      cmd = cmd.substr((N*N*6), cmd.length() - (N*N*6));
    }
    else if (cmd.substr(0, 8) == "!colors-" && cmd.length() >= 44)
    {
      cmd = cmd.substr(8, cmd.length() - 8);
      n = 1;
      for (int i = 1; i <= 6; i++)
      {
        clr[i] = "";
        for (int j = 0; j <= 5; j++)
        {
          if
          ((cmd.at(j) < '0' || cmd.at(j) > '9') &&
           (cmd.at(j) < 'A' || cmd.at(j) > 'F') &&
           (cmd.at(j) < 'a' || cmd.at(j) > 'f') )
            n = 0;
          clr[i] += cmd.at(j);
        }
        cmd = cmd.substr(6, cmd.length() - 6);
      }
      if (n)
        for (int i = 1; i <= 6; i++)
          col[i] = clr[i];
    }
    else if (cmd.substr(0, 13) == "!withgraphics")
    {
      disp_graphics = 1;
      cmd = cmd.substr(13, cmd.length() - 13);
    }
    else if (cmd.substr(0, 13) == "!withdiagrams")
    {
      disp_diagrams = 1;
      cmd = cmd.substr(13, cmd.length() - 13);
    }
    else if (cmd.substr(0, 7) == "!fixcol")
    {
      fixcol = 1;
      cmd = cmd.substr(7, cmd.length() - 7);
    }
    else if (cmd.substr(0, 4) == "!adv")
    {
      adv = 1;
      cmd = cmd.substr(4, cmd.length() - 4);
    }
    else if (cmd.substr(0, 5) == "!sel-" && cmd.length() >= 6)
    {
      if (cmd.at(5) >= '1' && cmd.at(5) <= '6')
        sel = (int)(cmd.at(5) - '0');
      cmd = cmd.substr(6, cmd.length() - 6);
    }
    else if (cmd.substr(0, 5) == "!help")
    {
      help = 1;
      cmd = cmd.substr(5, cmd.length() - 5);
    }
    else
    {
      cmd = "";
    }
  }
}

// main entry point
int main(int argc, char* argv[])
{
  string cmdln = "";
  if (argc >= 2) cmdln = argv[1];
  char f[256];
  int w;
  string s;
  FILE *fp;
  readcommandline(cmdln);
  printf("Content-Type: text/html\n\n");
  if (fp = fopen(html_template.c_str(), "r"))
  {
    while (fgets(f, sizeof(f)-1, fp))
    {
      s = f;
      if      ((w = s.find("%javadefaults%")) >= 0)
        java_defaults();
      else if ((w = s.find("%version%")) >= 0)
        printf("%s (%s)\n", version, mcube::version);
      else
        printf("%s", s.c_str());
    }
    fclose(fp);
  }
  else
  {
    printf("Missing %s template.  Please re-install.\n", html_template.c_str());
  }
  return 0;
}

//
